
#include<stdio.h>
#include<netinet/in.h>
#include<stdlib.h>

#include"sitar.h"
#include<almemsys/almemsys.h>


extern global_data_t gd;
extern swap_array_t sa;
extern struct scan_buf sb;


int m_exit(int);



int scan_malloc_set(private_data_t *pd)
{
int count;
count = 0;
/* port scan initialization bufer */
/*scan_buf */

#ifdef MALLOC_BUG
printf("MALLOC: scan_malloc_set():\n");
#endif

if((pd->sb.swap = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_stat = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_prot = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_host = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_port = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_port_h = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_port_m = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_timeout = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_timeout_h = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_timeout_m = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_data = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_info = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_free1 = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_free2 = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_free3 = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_free4 = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_tool = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_com = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.ch_plai = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.swap_2 = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((pd->sb.scan_all = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}


return 0;
}



int safree(int count)
{
int count_t;

#ifdef FREE_BUG
printf("FREE: safree():\n");
#endif

for(count_t = 0; count_t < gd.td.tbufmax; count_t ++){
   #ifdef XFREE_BUG
   printf("FREE: safree(): sa.swap_data[%d][%d] %s\n", count, count_t, sa.swap_data[count][count_t]);
   #endif
   free(sa.swap_data[count][count_t]);
   }
return 0;
}



int sacalloc(int count)
{
int count_t;

#ifdef MALLOC_BUG
printf("MALLOC: sacalloc():\n");
#endif

for(count_t = 0; count_t < gd.td.tbufmax; count_t ++){
   if((sa.swap_data[count][count_t]=(char *)calloc(2, sizeof(char)))==NULL){exit(1);}
   #ifdef XMALLOC_BUG
   printf("MALLOC: sacalloc(): sa.swap_data[%d][%d]\n", count, count_t);
   #endif
   }
return 0;
}



int swap_malloc_set(void)
{
int count;
int count_t;
count = 0;
count_t = 0;

#ifdef MALLOC_BUG
printf("MALLOC: swap_malloc_set():\n");
#endif

/* gd */

if((sa.count=(int*)malloc(sizeof(int)*(gd.td.tcount + 2)))==NULL){exit(1);}
if((sa.buf_err=(int*)malloc(sizeof(int)*(gd.td.tcount + 2)))==NULL){exit(1);}
if((sa.swap_data=(char***)malloc(sizeof(char**)*(gd.td.tcount + 2)))==NULL){exit(1);}

for(count = 0; count < gd.td.tcount; count++){
   sa.count[count] = 0;
   sa.buf_err[count] = 0;
   if((sa.swap_data[count]=(char**)malloc(sizeof(char*)*(gd.td.tbufmax + 2)))==NULL){exit(1);}
   sacalloc(count);
   }

if((sa.pt=(pthread_t *)malloc(sizeof(pthread_t)*(gd.td.tcount + 2)))==NULL){exit(1);}
if((sa.tmp_data=(char**)malloc(sizeof(char*)*(gd.td.tbufmax + 2)))==NULL){exit(1);}

for(count = 0; count < gd.td.tcount; count++){
   sa.pt[count] = 0;
   }

sa.main_buf_err = 0;

satmpcalloc();

return 0;
}



int satmpcalloc(void)
{
int count;

#ifdef MALLOC_BUG
printf("MALLOC: satmpcalloc():\n");
#endif

for(count = 0; count < gd.td.tbufmax; count++){
   if((sa.tmp_data[count]=(char *)calloc(2, sizeof(char)))==NULL){exit(1);}
   }
return 0;
}

int satmpfree(void)
{
int count;

#ifdef FREE_BUG
printf("FREE: satmpfree():\n");
#endif

for(count = 0; count < gd.td.tbufmax; count ++){
   free(sa.tmp_data[count]);
   }
return 0;
}



int swap_malloc_free(void)
{
int count;
int count_t;
count = 0;
count_t = 0;

#ifdef FREE_BUG
printf("FREE: swap_malloc_free():\n");
#endif

for(count = 0; count < gd.td.tcount; count ++){
   safree(count);
   free(sa.swap_data[count]);
   }
 
#ifdef FREE_BUG
printf("FREE: swap_malloc_free(): satmpfree()\n");
#endif
satmpfree();

free(sa.tmp_data);
free(sa.swap_data);
free(sa.count);
free(sa.buf_err);
free(sa.pt);

return 0;
}





int scan_malloc_free(private_data_t *pd)
{
/* int count;
count = 0; */
/* port scan initialization bufer */

#ifdef FREE_BUG
printf("FREE: scan_malloc_free():\n");
#endif

free(pd->sb.swap);
free(pd->sb.ch_stat);
free(pd->sb.ch_prot);
free(pd->sb.ch_host);
free(pd->sb.ch_port);
free(pd->sb.ch_port_h);
free(pd->sb.ch_port_m);
free(pd->sb.ch_timeout);
free(pd->sb.ch_timeout_h);
free(pd->sb.ch_timeout_m);
free(pd->sb.ch_data);
free(pd->sb.ch_info);
free(pd->sb.ch_free1);
free(pd->sb.ch_free2);
free(pd->sb.ch_free3);
free(pd->sb.ch_free4);
free(pd->sb.ch_tool);
free(pd->sb.ch_com);
free(pd->sb.ch_plai);
free(pd->sb.swap_2);
free(pd->sb.scan_all);
return 0;
}





int malloc_set(void)
{
/* initialization bufer */

#ifdef MALLOC_BUG
printf("MALLOC: malloc_set():\n");
#endif

/* file_path */
if((gd.fi.bat_filename = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.bat_filename_2 = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.bat_filestat = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.bat2_filename = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.bat2_filename_2 = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.bat2_filestat = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.command_path = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.fi.update_com = (char *)calloc(BUF_SLEN, sizeof(char)))==NULL){ m_exit(1);}


/* buff_set */
if((gd.bf.c_info = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.bf.c_name = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.bf.myfile = (char *)calloc(BUF_LEN, sizeof(char)))==NULL){ m_exit(1);}

/* my_sock */
if((gd.ip.ip_addr = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.ip.ip2_addr = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}

/* mail_data */
if((gd.smtp.ch_MailCc = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}

/* scan */
if((gd.sd.swap = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}
if((gd.sd.scan_all = (char *)calloc(IP_LEN, sizeof(char)))==NULL){ m_exit(1);}

return 0;
}




int malloc_set2(void)
{
int count;
count = 0;

#ifdef MALLOC_BUG
printf("MALLOC: malloc_set2():\n");
#endif

if((gd.td.fdstat=(int*)malloc(sizeof(int)*(gd.td.tcount + 2)))==NULL){exit(1);}
for(count = 0; count <= gd.td.tcount; count ++){
   gd.td.fdstat[count] = 0;
   }

if((gd.td.mutex_name=(char **)malloc(sizeof(char *) * (gd.td.tbufmax + 2)))==NULL){exit(1);}
for(count = 0; count <= gd.td.tbufmax; count++){
   if((gd.td.mutex_name[count]=(char *)calloc(2, sizeof(char)))==NULL){exit(1);}
   gd.td.mutex_name[count] = safe_memcpy(gd.td.mutex_name[count], " \n", BUF_MAX); 
   }

if((gd.td.middle_name=(char **)malloc(sizeof(char *) * (B_MAX + 2)))==NULL){exit(1);}
if((gd.td.middle_id=(char *)malloc(sizeof(char)*(B_MAX + 2)))==NULL){exit(1);}
for(count = 0; count <= B_MAX; count++){
   if((gd.td.middle_name[count]=(char *)calloc(2, sizeof(char)))==NULL){exit(1);}
   gd.td.middle_name[count] = safe_memcpy(gd.td.middle_name[count], " \n", BUF_MAX); 
   gd.td.middle_id[count] = 0;
   }

return 0;
}


int malloc_set2_free(void)
{
int count;

#ifdef FREE_BUG
printf("FREE: malloc_set2_free():\n");
#endif

for(count = 0; count <= gd.td.tbufmax; count++){
   free(gd.td.mutex_name[count]);
   }

for(count = 0; count <= B_MAX; count++){
   free(gd.td.mutex_name[count]);
   }

free(gd.td.fdstat);
free(gd.td.mutex_name);
free(gd.td.middle_name);
free(gd.td.middle_id);
return 0;
}




int malloc_set_free(void)
{

#ifdef FREE_BUG
printf("FREE: malloc_set_free():\n");
#endif

/* file_path */
free(gd.fi.bat_filename);
free(gd.fi.bat_filename_2);
free(gd.fi.bat_filestat);
free(gd.fi.bat2_filename);
free(gd.fi.bat2_filename_2);
free(gd.fi.bat2_filestat);
free(gd.fi.command_path);
free(gd.fi.update_com);

/* buff_set */
free(gd.bf.c_info);
free(gd.bf.c_name);
free(gd.bf.myfile);

/* my_sock */
free(gd.ip.ip_addr);
free(gd.ip.ip2_addr);

/* mail_data */
free(gd.smtp.ch_MailCc);

/* scan */
free(gd.sd.swap);
free(gd.sd.scan_all);

return 0;
}






int m_exit(int stat)
{
fprintf(stderr,"m_exit(): !!! The memory cannot be secured for the heap area. !!!\n");
exit(stat);
}





